//+------------------------------------------------------------------+
//|                                                          sms.mqh |
//|                                         Copyright 2013, Decanium |
//|                                          http://www.decanium.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2013, Decanium"
#property link      "http://www.decanium.com"
//+------------------------------------------------------------------+
//| Time label structure                                             |
//+------------------------------------------------------------------+
struct INCOMING_CTST_STR
  {
   datetime          time; // time
   int               gmt;  // time zone
  };
//+------------------------------------------------------------------+
//| SMS message structure                                            |
//+------------------------------------------------------------------+
struct INCOMING_SMS_STR
  {
   int               index;  // index in the modem memory
   string            sca;    // sender's SMS center number
   string            sender; // sender's number
   INCOMING_CTST_STR scts;   // SMS center time label
   string            text;   // text of the message
  };
//+------------------------------------------------------------------+
//| List of SMS messages                                             |
//+------------------------------------------------------------------+
struct SMS_LIST_STR
  {
   int               last;//last received SMS (or -1 if nothing has been received yet)
                          //(index in the list, not in the modem memory!)
   INCOMING_SMS_STR  list[SMS_LIST_SIZE];//list of SMS messages
  };

SMS_LIST_STR smsl;
//+------------------------------------------------------------------+
//| string number_enc(string num,bool sca)                           |
//| Coding the number in PDU                                         |
//| num - initial string                                             |
//| sca - what is coded (true - N of SMS center, false - N of sender)|
//| returning the coded number                                       |
//+------------------------------------------------------------------+
string number_enc(string num,bool sca)
  {
   string str,rstr;
   int i,len=0;
   int n=StringLen(num);
   if(StringSubstr(num,0,1)=="+")
     {//remove "+" before the number
      n--;
      str=StringSubstr(num,1,n);
     }
   else
      str=num;
//
   if(sca==false)
      len=n;//--- length in characters
   if((n%2)!=0)
     {//--- if the number of characters is odd, add "F" to the end of the string
      n++;
      str+="F";
     }
   if(sca==true)
      len=(n/2)+1;//length in bytes, including "number type" byte
//
   rstr = hex_to_str((uchar)len);
   rstr+=hex_to_str(0x91);//--- number type - international, dial type - ISDN   
   for(i=0;i<n;i+=2)
     {//---add the number, swapping tetrads
      rstr+= StringSubstr(str,i+1,1);
      rstr+= StringSubstr(str,i,1);
     }
   return(rstr);
  }
//+------------------------------------------------------------------+
//| bool number_dec(string &num,string str)                          |
//| decoding the number from PDU                                     |
//| str - source string                                              |
//| num - target string                                              |
//| returning true in case of success, otherwise - false             |
//+------------------------------------------------------------------+
bool number_dec(string &num,string str)
  {
   string s;
   int i,len=StringLen(str);
   if(len<2)
      return(false);
   uchar ntype=str_to_hex(StringSubstr(str,0,2));
   if((ntype&0x80)!=0x80)
      return(false);
   if(((ntype>>4)&0x07)==0x05)
     {//--- number type - alphanumeric
      num=decodePDU(StringSubstr(str,2,len-2));
     }
   else
     {
      num="+";
      for(i=0;i<(len-2);i+=2)
        {//--- add the number, swapping tetrads
         num+=StringSubstr(str,2+i+1,1);
         s=StringSubstr(str,2+i,1);
         if(s!="F")//leave the justifying F out of the number
            num+=s;
        }
     }
   return(true);
  }
//+------------------------------------------------------------------+
//| str_to_short                                                     |
//+------------------------------------------------------------------+
ushort str_to_short(string str)
  {
   ushort us=str_to_hex(StringSubstr(str,0,2));
   us=(us<<8)|str_to_hex(StringSubstr(str,2,2));
   return(us);
  }
//+------------------------------------------------------------------+
//| ParseSMS                                                         |
//+------------------------------------------------------------------+
bool ParseSMS(string str,INCOMING_SMS_STR &isms)
  {
   string strtmp="";
   int slen=StringLen(str);
   int i,pos=0;
   bool unicode=false;
   isms.sca="";
   isms.sender="";
   isms.scts.time= 0;
   isms.scts.gmt = 0;
   isms.text="";
//--- sca
   int len=str_to_hex(StringSubstr(str,pos,2));//in bytes
   pos+=2;
   if(pos>=slen)
      return(false);
   if(number_dec(strtmp,StringSubstr(str,pos,len*2))==true)
      isms.sca=strtmp;
   else
      return(false);
//--- pdu type
   pos+=len*2;
   if(pos>=slen)
      return(false);
   int tmp=str_to_hex(StringSubstr(str,pos,2));
   if((tmp&0x03)!=0)
     {
      //Print("Only accepted messages are analyzed");
      return(false);//only accepted messages are analyzed
     }
//--- oa
   pos+=2;
   if(pos>=slen)
      return(false);
   len=str_to_hex(StringSubstr(str,pos,2));//in characters
   if((len&0x01)==0x01)
      len++;//make the odd number even
   pos+=2;
   if(pos>=slen)
      return(false);
   if(number_dec(strtmp,StringSubstr(str,pos,len+2))==true)
      isms.sender=strtmp;
   else
      return(false);
//--- (leave out PID)
   pos+=len+2;
//--- DCS   
   pos+=2;
   if(pos>=slen)
      return(false);
   tmp=str_to_hex(StringSubstr(str,pos,2));
   if((tmp&0xE0)!=0)
     {
      //Print("Condensed text format and other odd features are not processed");
      return(false);//condensed text format and other odd features are not processed
     }
   if((tmp&0x0C)==0)
     {//--- default alphabet (7-bit)
      unicode=false;
     }
   else if((tmp&0x0C)==0x08)
     {//--- UCS2 (16bit Unicode)
      unicode=true;
     }
   else
     {
      Print("Unknown encoding, DCS="+DoubleToString(tmp,0));
      return(false);
     }
//--- SCTS
   pos+=2;
   if(pos>=slen)
      return(false);
   int timebuf[7];
   timebuf[0]=0;
   for(i=0;i<14;i+=2)
     {//fill timebuf, swapping tetrads
      timebuf[i>>1] = str_to_hex("0"+StringSubstr(str,pos+i,1));
      timebuf[i>>1]+= str_to_hex("0"+StringSubstr(str,pos+i+1,1))*10;
     }
   MqlDateTime time;
   time.year= 2000+timebuf[0];
   time.mon = timebuf[1];
   time.day = timebuf[2];
   time.hour= timebuf[3];
   time.min = timebuf[4];
   time.sec = timebuf[5];
   isms.scts.time= StructToTime(time);
   isms.scts.gmt = timebuf[6];
//       
   pos+=14;
   if(pos>=slen)
      return(false);
   tmp=str_to_hex(StringSubstr(str,pos,2));
   if(tmp==0)
     {
      isms.text="";
      return(true);//empty SMS 
     }
   pos+=2;
   if(pos>=slen)
      return(false);
//---
   if(unicode==true)
     {
      ushort us;
      for(i=0;i<(tmp*2);i+=4)
        {
         us=str_to_short(StringSubstr(str,pos+i,4));
         isms.text+=ShortToString(us);
        }
     }
   else
      isms.text=decodePDU(StringSubstr(str,pos,slen-pos));
//---
   return(true);
  }
//+------------------------------------------------------------------+
//| SendSMS                                                          |
//+------------------------------------------------------------------+
bool SendSMS(string da,string text,bool flash)
  {
   string str;
   uchar buf[176];
   int len,pdu_len;//
                   //
   if(modem.sms_sca=="" || da=="")
      return(false);
//--- SCA:
   str=number_enc(modem.sms_sca,true);//SMS center number
                                      //PDU:
   str+="11";//--- message lifetime format - relative, MTI = 01 (outgoing message)
//--- MR: 
   str+="00";//--- should be set to zero
//--- DA:
   string str_da=number_enc(da,false);//recipient's number
   pdu_len=StringLen(str_da);
   str+=str_da;
//--- PID:
   str+="00";
//--- DCS:
   if(flash==true)
      str+="F0";
   else
      str+="00";
//--- VP:
   str+="FF";//message lifetime - maximum (63 weeks)
             //UDL:
   len=StringLen(text);
   if(len>158)
      len=158;
   str+=hex_to_str((uchar)len);
//--- UD:
   if(len>0)
     {
      string userdata=encodePDU(text,false);
      pdu_len+=StringLen(userdata);
      str+=userdata;
     }
//---
   pdu_len=6+(pdu_len/2);
//---
   mdrv.sms_out_msg=str;
//---
   if(SendATcommand("AT+CMGS="+DoubleToString(pdu_len,0),at)==true)
     {
      if(at.res==1)
        {
         if(modem.sms_free_cnt!=-1 && modem.sms_free_cnt<inp_free_sms_daily)
            modem.sms_free_cnt++;
         //Print("SMS sent successfully");
         return(true);
        }
      else
        {
         //Print("Error when sending SMS");   
         return(false);
        }
     }
//else
//   Print("Error when sending SMS (the modem is not responding)");   
//
   return(false);
  }
//+------------------------------------------------------------------+
